{ An example of how to read a .GRP file (with limited error checking) }

CONST
  ID='By Samiel - samiel@fastlane.net'; {That's me}

TYPE
  Str12=array [1..12] of byte; {12 byte array}

  Header=
    record
      KS:Str12; {KenSilverman}
      NumGrp:longint; {Number of files contained in the .GRP file}
    end;

  GrpRec=
    record
      FN:Str12; {FILENAME}
      Len:longint; {Length of the file}
    end;

VAR
  count:longint; {Counter}
  Head:Header;   {Header}
  Grp:GrpRec;    {Record}
  F:File;        {File variable}
  IsGrp:boolean; {.GRP file verification flag}
  Res:word;      {Result of BlockRead}
  Exist:boolean; {File exist flag}
  tmp:string;    {String}

Procedure Error(s:string;var F:File); {Halts the program on an error}
Begin
  writeln;
  writeln('Error : ',s);
  writeln;
  {$I-}
  close(F); {If file F is open, then close it}
  {$I+}
  Halt;     {Halt the program}
End;

Function Str12ToStr(s:str12):string; {Convert a 12 byte array to a string}
Var
  j:byte;
  tmp:string;
Begin
  tmp:='';
  for j:=1 to 12 do
    tmp:=tmp+chr(s[j]); {Add the character}
  Str12ToStr:=tmp;
End;

BEGIN
  writeln;
  writeln(ID); {Write an ID}
  if paramcount=1 then {Make sure there is a parameter}
    begin
      tmp:=paramstr(1); {Get the parameter}
      for count:=1 to length(tmp) do
        tmp[count]:=Upcase(tmp[count]); {Convert it to uppercase}

      {$I-}
      assign(F,tmp);
      reset(F);
      close(F);
      {$I+}
      Exist:=(IOResult=0) and (tmp<>''); {Check to see if the file exists}

      if Not Exist then
        Error('File '+tmp+' Not Found',F) {If not, halt the program}
      else 
        begin
          assign(F,tmp);
          reset(F,1); {Read BYTE sized records}

          BlockRead(F,Head,SizeOf(Head),Res); {Read the header}
          if Res<>SizeOf(Head) then
            Error('Unexpected End Of File',F); {Make sure it's good}

          IsGrp:=true;
          tmp:='KenSilverman';
          for count:=1 to 12 do {Determine if it is a .GRP file}
            if tmp[count]<>chr(Head.KS[count]) then
              IsGrp:=false;

          if not IsGrp then
            Error(tmp+' Not A Group File',F); {If not, halt the program}

          count:=0;

          writeln;

          repeat
            inc(count); {Increment the file number}

            BlockRead(F,Grp,SizeOf(Grp),Res); {Read the record}
            if Res<>SizeOf(Head) then
              Error('Unexpected End Of File',F); {Make sure it's good}

            writeln(Str12ToStr(Grp.FN),' - ',Grp.Len:1,' bytes'); {Write it}
          until count=Head.NumGrp; {Until we reach the number specified}

          writeln;

          close(F); {Close the file}
        end;
    end
  else
    begin
      writeln;
      writeln('USAGE:'); {How to use GRPINFO}
      writeln;
      writeln('  GRPINFO <GROUPFILE.GRP>');
    end;
END.
